@@ -2,7 +2,7 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
-//! Custom derives for `Yokeable` and `ZeroCopyFrom` from the `yoke` crate.
+//! Custom derives for `Yokeable` from the `yoke` crate.
 
 use proc_macro::TokenStream;
 use proc_macro2::{Span, TokenStream as TokenStream2};
@@ -34,7 +34,17 @@ pub fn yokeable_derive(input: TokenStream) -> TokenStream {
 }
 
 fn yokeable_derive_impl(input: &DeriveInput) -> TokenStream2 {
-    let tybounds = input.generics.type_params().collect::<Vec<_>>();
+    let tybounds = input
+        .generics
+        .type_params()
+        .map(|ty| {
+            // Strip out param defaults, we don't need them in the impl
+            let mut ty = ty.clone();
+            ty.eq_token = None;
+            ty.default = None;
+            ty
+        })
+        .collect::<Vec<_>>();
     let typarams = tybounds
         .iter()
         .map(|ty| ty.ident.clone())
@@ -120,11 +130,16 @@ fn yokeable_derive_impl(input: &DeriveInput) -> TokenStream2 {
                             yoke_bounds.push(parse_quote!(#fty: yoke::Yokeable<'a, Output = #fty>));
                         }
                     }
-                    // By calling transform_owned on all fields, we manually prove
-                    // that the lifetimes are covariant, since this requirement
-                    // must already be true for the type that implements transform_owned().
-                    quote! {
-                        <#fty as yoke::Yokeable<'a>>::transform_owned(#field)
+                    if has_ty || has_lt {
+                        // By calling transform_owned on all fields, we manually prove
+                        // that the lifetimes are covariant, since this requirement
+                        // must already be true for the type that implements transform_owned().
+                        quote! {
+                            <#fty as yoke::Yokeable<'a>>::transform_owned(#field)
+                        }
+                    } else {
+                        // No nested lifetimes, so nothing to be done
+                        quote! { #field }
                     }
                 })
             });
@@ -207,130 +222,6 @@ fn yokeable_derive_impl(input: &DeriveInput) -> TokenStream2 {
     }
 }
 
-/// Custom derive for `yoke::ZeroCopyFrom`,
-///
-/// This implements `ZeroCopyFrom<Ty> for Ty` for types
-/// without a lifetime parameter, and `ZeroCopyFrom<Ty<'data>> for Ty<'static>`
-/// for types with a lifetime parameter.
-///
-/// Apply the `#[yoke(cloning_zcf)]` attribute if you wish for this custom derive
-/// to use `.clone()` for its implementation. The attribute can be applied to
-/// fields as well.
-#[proc_macro_derive(ZeroCopyFrom, attributes(yoke))]
-pub fn zcf_derive(input: TokenStream) -> TokenStream {
-    let input = parse_macro_input!(input as DeriveInput);
-    TokenStream::from(zcf_derive_impl(&input))
-}
-
-fn has_cloning_zcf_attr(attrs: &[syn::Attribute]) -> bool {
-    attrs.iter().any(|a| {
-        if let Ok(i) = a.parse_args::<Ident>() {
-            if i == "cloning_zcf" {
-                return true;
-            }
-        }
-        false
-    })
-}
-
-fn zcf_derive_impl(input: &DeriveInput) -> TokenStream2 {
-    let tybounds = input.generics.type_params().collect::<Vec<_>>();
-    let typarams = tybounds
-        .iter()
-        .map(|ty| ty.ident.clone())
-        .collect::<Vec<_>>();
-    let has_clone = has_cloning_zcf_attr(&input.attrs);
-    let lts = input.generics.lifetimes().count();
-    let name = &input.ident;
-    if lts == 0 {
-        let (clone, clone_trait) = if has_clone {
-            (quote!(this.clone()), quote!(Clone))
-        } else {
-            (quote!(*this), quote!(Copy))
-        };
-        let bounds: Vec<WherePredicate> = typarams
-            .iter()
-            .map(|ty| parse_quote!(#ty: #clone_trait + 'static))
-            .collect();
-        quote! {
-            impl<'zcf, #(#tybounds),*> yoke::ZeroCopyFrom<'zcf, #name<#(#typarams),*>> for #name<#(#typarams),*> where #(#bounds),* {
-                fn zero_copy_from(this: &'zcf Self) -> Self {
-                    #clone
-                }
-            }
-        }
-    } else {
-        if lts != 1 {
-            return syn::Error::new(
-                input.generics.span(),
-                "derive(ZeroCopyFrom) cannot have multiple lifetime parameters",
-            )
-            .to_compile_error();
-        }
-        if has_clone {
-            return quote! {
-                impl<'zcf> yoke::ZeroCopyFrom<'zcf, #name<'_>> for #name<'zcf> {
-                    fn zero_copy_from(this: &'zcf #name<'_>) -> Self {
-                        this.clone()
-                    }
-                }
-            };
-        }
-
-        let structure = Structure::new(input);
-        let generics_env = typarams.iter().cloned().collect();
-
-        let mut zcf_bounds: Vec<WherePredicate> = vec![];
-        let body = structure.each_variant(|vi| {
-            let variant_cloning = has_cloning_zcf_attr(vi.ast().attrs);
-            vi.construct(|f, i| {
-                let binding_cloning = variant_cloning || has_cloning_zcf_attr(&f.attrs);
-                let binding = format!("__binding_{}", i);
-                let field = Ident::new(&binding, Span::call_site());
-
-                if binding_cloning {
-                    quote! {
-                        #field.clone()
-                    }
-                } else {
-                    let fty = replace_lifetime(&f.ty, custom_lt("'zcf"));
-                    let lifetime_ty = replace_lifetime(&f.ty, custom_lt("'zcf_inner"));
-
-                    let (has_ty, has_lt) = visitor::check_type_for_parameters(&f.ty, &generics_env);
-                    if has_ty {
-                        // For types without type parameters, the compiler can figure out that the field implements
-                        // ZeroCopyFrom on its own. However, if there are type parameters, there may be complex preconditions
-                        // to `FieldTy: ZeroCopyFrom` that need to be satisfied. We get them to be satisfied by requiring
-                        // `FieldTy<'zcf>: ZeroCopyFrom<'zcf, FieldTy<'zcf_inner>>`
-                        if has_lt {
-                            zcf_bounds
-                                .push(parse_quote!(#fty: yoke::ZeroCopyFrom<'zcf, #lifetime_ty>));
-                        } else {
-                            zcf_bounds.push(parse_quote!(#fty: yoke::ZeroCopyFrom<'zcf, #fty>));
-                        }
-                    }
-
-                    // By doing this we essentially require ZCF to be implemented
-                    // on all fields
-                    quote! {
-                        <#fty as yoke::ZeroCopyFrom<'zcf, #lifetime_ty>>::zero_copy_from(#field)
-                    }
-                }
-            })
-        });
-
-        quote! {
-            impl<'zcf, 'zcf_inner, #(#tybounds),*> yoke::ZeroCopyFrom<'zcf, #name<'zcf_inner, #(#typarams),*>> for #name<'zcf, #(#typarams),*>
-                where
-                #(#zcf_bounds,)* {
-                fn zero_copy_from(this: &'zcf #name<'zcf_inner, #(#typarams),*>) -> Self {
-                    match *this { #body }
-                }
-            }
-        }
-    }
-}
-
 fn static_lt() -> Lifetime {
     Lifetime::new("'static", Span::call_site())
 }
